@@ -21,72 +21,50 @@
 #include "zm_ffmpeg.h"
 #include "zm_image.h"
 #include "zm_logger.h"
-#include "zm_signal.h"
 
-#if HAVE_QUADRA
-#include "libavutil/hwcontext_ni_quad.h"
-#endif
+extern "C" {
+#include <libavutil/pixdesc.h>
+}
 
 using namespace std;
 AVPixelFormat target_format = AV_PIX_FMT_NONE;
 
 ZMPacket::ZMPacket() :
-  //lck_(mutex_, std::defer_lock),
   locked(false),
   keyframe(0),
   stream(nullptr),
-  //packet({}),
-  /*
-  in_frame({}),
-  out_frame({}),
-  ai_frame({}),
-  hw_frame({}),
-  */
-  //timestamp({}),
   image(nullptr),
   y_image(nullptr),
   analysis_image(nullptr),
-  ai_image(nullptr),
   score(-1),
   codec_type(AVMEDIA_TYPE_UNKNOWN),
   image_index(-1),
   codec_imgsize(0),
   pts(0),
   decoded(false),
-  analyzed(false)
-{
+  analyzed(false) {
   packet = av_packet_ptr{av_packet_alloc()};
 }
 
 ZMPacket::ZMPacket(Image *i, SystemTimePoint tv) :
-  //lck_(mutex_, std::defer_lock),
   locked(false),
   keyframe(0),
   stream(nullptr),
-  /*
-  in_frame({}),
-  out_frame({}),
-  ai_frame({}),
-  hw_frame({}),
-  */
   timestamp(tv),
   image(i),
   y_image(nullptr),
   analysis_image(nullptr),
-  ai_image(nullptr),
   score(-1),
   codec_type(AVMEDIA_TYPE_UNKNOWN),
   image_index(-1),
   codec_imgsize(0),
   pts(0),
   decoded(false),
-  analyzed(false)
-{
+  analyzed(false) {
   packet = av_packet_ptr{av_packet_alloc()};
 }
 
 ZMPacket::ZMPacket(ZMPacket &p) :
-  //lck_(mutex_, std::defer_lock),
   locked(false),
   keyframe(p.keyframe),
   stream(p.stream),
@@ -94,32 +72,24 @@ ZMPacket::ZMPacket(ZMPacket &p) :
   image(p.image),
   y_image(p.y_image),
   analysis_image(p.analysis_image),
-  ai_image(p.ai_image),
   score(p.score),
   codec_type(AVMEDIA_TYPE_UNKNOWN),
   image_index(p.image_index),
   codec_imgsize(0),
   pts(p.pts),
   decoded(p.decoded),
-  analyzed(p.analyzed)
-{
+  analyzed(p.analyzed) {
   packet = av_packet_ptr{av_packet_alloc()};
 
-  Error("Packet copy");
   if (av_packet_ref(packet.get(), p.packet.get()) < 0) {
     Error("error refing packet");
   }
 }
 
 ZMPacket::~ZMPacket() {
-  //Debug(1, "Deleting packet %d", image_index);
   delete analysis_image;
-  if (image) delete image;
-  if (y_image) delete y_image;
-  if (ai_image) delete ai_image;
-  if (hw_frame) {
-    Debug(1, "Should not have hw_frame in destructor %d", image_index);
-  }
+  delete image;
+  delete y_image;
 }
 
 ssize_t ZMPacket::ram() {
@@ -130,76 +100,71 @@ ssize_t ZMPacket::ram() {
          (analysis_image ? analysis_image->Size() : 0);
 }
 
+int ZMPacket::send_packet(AVCodecContext *ctx) {
+  // ret == 0 means EAGAIN
+  int ret = avcodec_send_packet(ctx, packet.get());
+  if (ret < 0) {
+    if (ret == AVERROR(EAGAIN)) {
+      Debug(1, "Unable to send packet %d %s, packet %d", ret, av_make_error_string(ret).c_str(), image_index);
+      //ret = avcodec_send_packet(ctx, packet.get());
+      return 0;
+    } else {
+      Error("Unable to send packet %d %s, packet %d", ret, av_make_error_string(ret).c_str(), image_index);
+      return ret;
+    }
+  }
+  Debug(3, "Ret from send_packet %d %s, packet %d", ret, av_make_error_string(ret).c_str(), image_index);
+  return 1;
+}
+
+/* Returns < 0 on error, 0 for go again, > 0 for success */
 int ZMPacket::receive_frame(AVCodecContext *ctx) {
   av_frame_ptr receive_frame{av_frame_alloc()};
   if (!receive_frame) {
     Error("Error allocating frame");
-    return 0;
+    return -1;
   }
   int ret = avcodec_receive_frame(ctx, receive_frame.get());
-  Debug(1, "Ret from receive_frame ret: %d %s, packet %d", ret, av_make_error_string(ret).c_str(), image_index);
-  if (ret == AVERROR(EAGAIN)) {
-    return 0;
-  } else if (ret < 0) {
-    Error("Ret from receive_frame ret: %d %s, packet %d", ret, av_make_error_string(ret).c_str(), image_index);
-    return ret;
+  if (ret < 0) {
+    if (ret == AVERROR(EAGAIN)) {
+      Debug(1, "Ret from receive_frame ret: %d %s, packet %d", ret, av_make_error_string(ret).c_str(), image_index);
+      return 0;
+    } else {
+      Error("Ret from receive_frame ret: %d %s, packet %d", ret, av_make_error_string(ret).c_str(), image_index);
+      return ret;
+    }
   }
 
-  in_frame = std::move(receive_frame);
-  //zm_dump_video_frame(in_frame.get(), "got frame");
-#if HAVE_QUADRA
-  AVFrame * frame = in_frame.get();
-  niFrameSurface1_t *pfs = (niFrameSurface1_t *) frame->data[3];
-  Debug(1, "*** Receive frame bufid %d (%d %d) ***", (pfs ? pfs->ui16FrameIdx : -1),
-      av_buffer_get_ref_count(frame->buf[0]),
-      av_frame_is_writable(frame));
+  // For hardware frames, do the transfer immediately while the context is valid
+  // The nvidia-vaapi-driver can have issues if there's a delay between decode and transfer
+#if HAVE_LIBAVUTIL_HWCONTEXT_H
+#if LIBAVCODEC_VERSION_CHECK(57, 89, 0, 89, 0)
+  if (receive_frame->hw_frames_ctx) {
+    Debug(2, "Hardware frame received, transferring immediately");
+    av_frame_ptr sw_frame{av_frame_alloc()};
+    ret = av_hwframe_transfer_data(sw_frame.get(), receive_frame.get(), 0);
+    if (ret < 0) {
+      Error("Immediate hw transfer failed: %s, packet %d", av_make_error_string(ret).c_str(), image_index);
+      return ret;
+    }
+    ret = av_frame_copy_props(sw_frame.get(), receive_frame.get());
+    if (ret < 0) {
+      Warning("Failed to copy frame props: %s", av_make_error_string(ret).c_str());
+    }
+    // Release GPU surface immediately - we have the software frame now
+    // Keeping hw_frame would hold GPU memory and exhaust the surface pool
+    // receive_frame goes out of scope here and releases the surface
+    in_frame = std::move(sw_frame);
+    zm_dump_video_frame(in_frame.get(), "After immediate hwtransfer");
+    return 1;
+  }
 #endif
+#endif
+  in_frame = std::move(receive_frame);
 
   return 1;
 }  // end int ZMPacket::receive_frame(AVCodecContext *ctx)
 
-int ZMPacket::send_packet(AVCodecContext *ctx) {
-  // ret == 0 means EAGAIN
-  // We only send a packet if we have a delayed_packet, otherwise packet is the delayed_packet
-  int ret = avcodec_send_packet(ctx, packet.get());
-  if (ret == AVERROR(EAGAIN)) {
-    Debug(2, "Unable to send packet %d %s, packet %d", ret, av_make_error_string(ret).c_str(), image_index);
-    //ret = avcodec_send_packet(ctx, packet.get());
-    return 0;
-  }
-  if (ret < 0) {
-    Error("Unable to send packet %d %s, packet %d", ret, av_make_error_string(ret).c_str(), image_index);
-    return ret;
-  }
-  Debug(1, "Ret from send_packet %d %s, packet %d", ret, av_make_error_string(ret).c_str(), image_index);
-  return 1;
-}
-
-/* returns < 0 on error, 0 on not ready, int bytes consumed on success
- * This functions job is to populate in_frame with the image in an appropriate
- * format. It MAY also populate image if able to.  In this case in_frame is populated
- * by the image buffer.
- */
-int ZMPacket::decode(AVCodecContext *ctx, std::shared_ptr<ZMPacket> delayed_packet) {
-  Debug(4, "about to decode video, image_index is (%d)", image_index);
-
-  int ret = send_packet(ctx);
-
-  av_frame_ptr receive_frame{av_frame_alloc()};
-  ret = avcodec_receive_frame(ctx, receive_frame.get());
-  Debug(1, "Ret from receive_frame %d %s", ret, av_make_error_string(ret).c_str());
-  if (ret == AVERROR(EAGAIN)) {
-    return 0;
-  } else if (ret < 0) {
-    return ret;
-  }
-
-  delayed_packet->in_frame = std::move(receive_frame);
-  zm_dump_video_frame(delayed_packet->in_frame.get(), "got frame");
-  //delayed_packet->get_hwframe(ctx);
-  return 1;
-}
-
 bool ZMPacket::needs_hw_transfer(AVCodecContext *ctx) {
   if (!(ctx && in_frame.get())) {
     Error("No ctx %p or in_frame %p", ctx, in_frame.get());
@@ -207,6 +172,10 @@ bool ZMPacket::needs_hw_transfer(AVCodecContext *ctx) {
   }
 #if HAVE_LIBAVUTIL_HWCONTEXT_H
 #if LIBAVCODEC_VERSION_CHECK(57, 89, 0, 89, 0)
+  // If frame has no hw_frames_ctx, it's already a software frame
+  if (!in_frame->hw_frames_ctx) {
+    return false;
+  }
   if (
       (ctx->sw_pix_fmt != AV_PIX_FMT_NONE)
       and
@@ -219,10 +188,11 @@ bool ZMPacket::needs_hw_transfer(AVCodecContext *ctx) {
   return false;
 }
 
-int ZMPacket::get_hwframe(AVCodecContext *ctx) {
+int ZMPacket::transfer_hwframe(AVCodecContext *ctx) {
   if (hw_frame) {
-    Error("Already have hw_frame in get_hwframe");
-    return 0;
+    // Already transferred in receive_frame
+    Debug(2, "Hardware frame already transferred");
+    return 1;
   }
 #if HAVE_LIBAVUTIL_HWCONTEXT_H
 #if LIBAVCODEC_VERSION_CHECK(57, 89, 0, 89, 0)
@@ -237,44 +207,48 @@ int ZMPacket::get_hwframe(AVCodecContext *ctx) {
         av_get_pix_fmt_name(static_cast<AVPixelFormat>(in_frame->format))
         );
 
+    // Frame gets moved no matter what
+    hw_frame = std::move(in_frame);
+    zm_dump_video_frame(hw_frame.get(), "Before hwtransfer");
+
+    // Verify hw_frames_ctx is valid before attempting transfer
+    if (!hw_frame->hw_frames_ctx) {
+      Error("Hardware frame has no hw_frames_ctx, cannot transfer");
+      hw_frame = nullptr;
+      in_frame = nullptr;
+      return -1;
+    }
+
     av_frame_ptr new_frame{av_frame_alloc()};
+    // Don't set format - let FFmpeg use the hw_frames_ctx default format
+    // (nvidia-vaapi-driver only supports nv12/p010 transfer, not yuvj420p)
+    // The later swscale conversion will handle format conversion
+
     /* retrieve data from GPU to CPU */
-    int ret = av_hwframe_transfer_data(new_frame.get(), in_frame.get(), 0);
+    int ret = av_hwframe_transfer_data(new_frame.get(), hw_frame.get(), 0);
     if (ret < 0) {
-      Error("Unable to transfer frame: %s, continuing", av_make_error_string(ret).c_str());
+      Error("Unable to transfer frame: %s", av_make_error_string(ret).c_str());
+      hw_frame = nullptr;
+      in_frame = nullptr;
       return ret;
     }
-    
-    ret = av_frame_copy_props(new_frame.get(), in_frame.get());
+
+    ret = av_frame_copy_props(new_frame.get(), hw_frame.get());
     if (ret < 0) {
       Error("Unable to copy props: %s, continuing", av_make_error_string(ret).c_str());
     }
 
-    hw_frame = std::move(in_frame);
-    zm_dump_video_frame(hw_frame.get(), "Before hwtransfer");
-
     in_frame = std::move(new_frame);
     zm_dump_video_frame(in_frame.get(), "After hwtransfer");
   } else
-#endif
-#endif
     Debug(3, "Same pix format %s so not hwtransferring. sw_pix_fmt is %s",
         av_get_pix_fmt_name(ctx->pix_fmt),
         av_get_pix_fmt_name(ctx->sw_pix_fmt)
         );
+#endif
+#endif
   return 1;
-} // end ZMPacket::get_hwframe
-
-Image *ZMPacket::get_ai_image() {
-  if (!ai_frame) {
-    Error("Can't get image without ai frame.. maybe need to decode first");
-    return nullptr;
-  }
-  if (!ai_image) {
-    ai_image = new Image(ai_frame.get());
-  }
-  return ai_image;
-}
+} // end ZMPacket::transfer_hwframe
 
 Image *ZMPacket::get_image(Image *i) {
   if (!in_frame) {
@@ -282,14 +256,13 @@ Image *ZMPacket::get_image(Image *i) {
     return nullptr;
   }
   if (!image) {
-    if (i) {
-      image = i;
-      image->Assign(in_frame.get());
-    } else {
-      // What if it's the hwframe?
-      image = new Image(in_frame.get());
+    if (!i) {
+      Error("Need a pre-allocated image buffer");
+      return nullptr;
     }
+    image = i;
   }
+  image->Assign(in_frame.get());
   return image;
 }
 
@@ -300,13 +273,36 @@ Image *ZMPacket::set_image(Image *i) {
 
 Image *ZMPacket::get_y_image() {
   if (!y_image) {
+    if (!in_frame) {
+      Error("Can't get y_image without frame, maybe need to decode first");
+      return nullptr;
+    }
+
+    // Check if the pixel format has a Y channel accessible in data[0]
+    // This requires a planar YUV format (not RGB, not packed YUV)
+    const AVPixFmtDescriptor *desc = av_pix_fmt_desc_get(static_cast<AVPixelFormat>(in_frame->format));
+    if (!desc) {
+      Error("Unable to get pixel format descriptor for format %d", in_frame->format);
+      return nullptr;
+    }
+
+    // Must not be RGB (no Y channel) and must be planar (Y is in data[0])
+    if (desc->flags & AV_PIX_FMT_FLAG_RGB) {
+      Error("Cannot get Y image from RGB format %s", desc->name);
+      return nullptr;
+    }
+    if (!(desc->flags & AV_PIX_FMT_FLAG_PLANAR)) {
+      Error("Cannot get Y image from non-planar format %s (Y is interleaved)", desc->name);
+      return nullptr;
+    }
+
     y_image = new Image(in_frame->width, in_frame->height, 1, ZM_SUBPIX_ORDER_NONE, in_frame->data[0], 0, 0);
   }
   return y_image;
 }
 
 AVPacket *ZMPacket::set_packet(AVPacket *p) {
-  if (av_packet_ref(packet.get(), p) < 0) {
+  if (zm_av_packet_ref(packet.get(), p) < 0) {
     Error("error refing packet");
   }
 
@@ -315,31 +311,6 @@ AVPacket *ZMPacket::set_packet(AVPacket *p) {
   return packet.get();
 }
 
-void ZMPacket::set_ai_frame(AVFrame *frame) {
-  ai_frame = av_frame_ptr{frame};
-}
-
-int ZMPacket::get_hw_device_id() const {
-  int deviceid = -1;
-#if HAVE_QUADRA
-  if (hw_frame && hw_frame->format == AV_PIX_FMT_NI_QUAD) {
-    deviceid = ni_get_cardno(hw_frame.get());
-  }
-#endif
-  return deviceid;
-}
-
-AVFrame *ZMPacket::get_ai_frame() {
-  if (!ai_frame) {
-    ai_frame = av_frame_ptr{av_frame_alloc()};
-    if (!ai_frame) {
-      Error("Unable to allocate a frame");
-      return nullptr;
-    }
-  }
-  return ai_frame.get();
-} // end AVFrame *ZMPacket::get_ai_frame( AVCodecContext *ctx );
-
 AVFrame *ZMPacket::get_out_frame(int width, int height, AVPixelFormat format) {
   if (!out_frame) {
     out_frame = av_frame_ptr{av_frame_alloc()};
@@ -348,21 +319,17 @@ AVFrame *ZMPacket::get_out_frame(int width, int height, AVPixelFormat format) {
       return nullptr;
     }
 
-    out_frame->width = width;
-    out_frame->height = height;
-    out_frame->format = format;
+    int alignment = 32;
+    if (width%alignment) alignment = 1;
 
-    int ret = av_frame_get_buffer(out_frame.get(), 0);
-#if 0
     codec_imgsize = av_image_get_buffer_size(format, width, height, alignment);
     Debug(1, "buffer size %u from %s %dx%d", codec_imgsize, av_get_pix_fmt_name(format), width, height);
     out_frame->buf[0] = av_buffer_alloc(codec_imgsize);
-#endif
-    if (ret) {
+    if (!out_frame->buf[0]) {
       Error("Unable to allocate a frame buffer");
+      out_frame = nullptr;
       return nullptr;
     }
-#if 0
     int ret;
     if ((ret=av_image_fill_arrays(
                out_frame->data,
@@ -376,7 +343,10 @@ AVFrame *ZMPacket::get_out_frame(int width, int height, AVPixelFormat format) {
       out_frame = nullptr;
       return nullptr;
     }
-#endif
+
+    out_frame->width = width;
+    out_frame->height = height;
+    out_frame->format = format;
   }
   return out_frame.get();
 } // end AVFrame *ZMPacket::get_out_frame( AVCodecContext *ctx );
@@ -413,7 +383,11 @@ void ZMPacket::unlock(std::unique_lock<std::mutex> &lck_) {
 };
 
 void ZMPacket::unlock() {
-  Debug(4, "packet %d unlocked, %p, locked %d, owns %d", image_index, this, locked, our_lck_.owns_lock());
-  our_lck_.unlock();
-};
+  if (locked) {
+    Debug(4, "packet %d unlocked, %p, locked %d, owns %d", image_index, this, locked, our_lck_.owns_lock());
+    our_lck_.unlock();
+  } else {
+    Error("Attempt to unlock already unlocked packet %d unlocked, %p, locked %d, owns %d", image_index, this, locked, our_lck_.owns_lock());
+  }
+}
 
