@@ -25,6 +25,9 @@
 #include "zm_packet.h"
 #include "zm_signal.h"
 
+#include <climits>
+#include <vector>
+
 PacketQueue::PacketQueue():
   video_stream_id(-1),
   max_video_packet_count(-1),
@@ -36,7 +39,8 @@ PacketQueue::PacketQueue():
   warned_count(0),
   has_out_of_order_packets_(false),
   max_keyframe_interval_(0),
-  frames_since_last_keyframe_(0)
+  frames_since_last_keyframe_(0),
+  next_queue_index_(0)
 {
 }
 
@@ -44,27 +48,23 @@ PacketQueue::PacketQueue():
  * Assumes first stream added will be the video stream
  */
 int PacketQueue::addStream() {
+  std::lock_guard<std::mutex> lck(mutex);
   deleting = false;
   if (max_stream_id == -1) {
     video_stream_id = 0;
     max_stream_id = 0;
   } else {
-    max_stream_id ++;
+    max_stream_id++;
   }
 
-  if (packet_counts) delete[] packet_counts;
-  packet_counts = new int[max_stream_id+1];
-  for (int i=0; i <= max_stream_id; ++i)
+  packet_counts.reset(new int[max_stream_id + 1]);
+  for (int i = 0; i <= max_stream_id; ++i)
     packet_counts[i] = 0;
   return max_stream_id;
 }
 
 PacketQueue::~PacketQueue() {
   clear();
-  if (packet_counts) {
-    delete[] packet_counts;
-    packet_counts = nullptr;
-  }
   while (!iterators.empty()) {
     packetqueue_iterator *it = iterators.front();
     iterators.pop_front();
@@ -79,6 +79,9 @@ PacketQueue::~PacketQueue() {
  */
 
 bool PacketQueue::queuePacket(std::shared_ptr<ZMPacket> add_packet) {
+  std::unique_lock<std::mutex> lck(mutex);
+  if (deleting or zm_terminate) return false;
+
   if (iterators.empty()) {
     Debug(4, "No iterators so no one needs us to queue packets.");
     return false;
@@ -87,9 +90,6 @@ bool PacketQueue::queuePacket(std::shared_ptr<ZMPacket> add_packet) {
     Debug(4, "No video keyframe so no one needs us to queue packets.");
     return false;
   }
-  {
-    std::unique_lock<std::mutex> lck(mutex);
-    if (deleting or zm_terminate) return false;
 
     AVPacket *add_avpacket = add_packet->packet.get();  // because std::shared_ptr accesses are way more expensive
     if (!has_out_of_order_packets_ and (add_avpacket->dts != AV_NOPTS_VALUE)) {
@@ -119,13 +119,14 @@ bool PacketQueue::queuePacket(std::shared_ptr<ZMPacket> add_packet) {
           Debug(1, "Have new keyframe interval %d", max_keyframe_interval_);
         }
       } else {
-        frames_since_last_keyframe_ = 1;
+        frames_since_last_keyframe_ = 0;
         if (!max_keyframe_interval_) max_keyframe_interval_ = 1;
       }
     } else {
       Debug(1, "Not video stream %d", add_avpacket->stream_index);
     }
 
+    add_packet->queue_index = next_queue_index_++;
     pktQueue.push_back(add_packet);
 
     /* Any iterators that are pointing to the end will now point to the newly pushed packet */
@@ -158,35 +159,27 @@ bool PacketQueue::queuePacket(std::shared_ptr<ZMPacket> add_packet) {
                 , max_video_packet_count, packet_counts[video_stream_id], max_keyframe_interval_);
       }
 
+      // Going to delete a GOP of packets. So find another keyframe and delete everything before it.
       auto it = pktQueue.begin();
       // Start at second packet because the first is always a keyframe unless we don't care about keyframes
       it ++;
       while ((*it != add_packet) && !(deleting or zm_terminate)) {
         std::shared_ptr <ZMPacket>zm_packet = *it;
-         ZMPacketLock packet_lock(zm_packet);
 
-        if (!packet_lock.trylock()) {
-          if (warned_count < 2) {
-            warned_count++;
-            Warning("Found locked packet %d when trying to free up video packets. Our packet %d", zm_packet->image_index, add_packet->image_index);
-            break;
-          }
-        }
         if (zm_packet->packet->stream_index == video_stream_id and zm_packet->keyframe) {
           for ( it = pktQueue.begin(); *it != zm_packet; ) {
             it = this->deletePacket(it);
           }
           break;
-        }
+        }  // end if erasing a whole gop
         ++it;
       }  // end foreach
     } else if (warned_count > 0) {
       warned_count--;
     }  // end if not able catch up
-    // We signal on every packet because someday we may analyze sound
-    Debug(4, "packetqueue queuepacket, unlocked signalling");
-    condition.notify_all();
-  }  // end lock scope
+  // We signal on every packet because someday we may analyze sound
+  Debug(4, "packetqueue queuepacket, unlocked signalling");
+  condition.notify_all();
 
   return true;
 }  // end bool PacketQueue::queuePacket(ZMPacket* zm_packet)
@@ -223,12 +216,18 @@ packetqueue_iterator PacketQueue::deletePacket(packetqueue_iterator it) {
 }
 
 void PacketQueue::clearPackets(const std::shared_ptr<ZMPacket> &add_packet) {
+  // Packets removed from the queue are collected here and destroyed after the
+  // mutex is released, so that expensive ZMPacket destructors (freeing images,
+  // av packets) don't run while the queue is locked.
+  std::vector<std::shared_ptr<ZMPacket>> packets_to_destroy;
+
+  {
+  std::unique_lock<std::mutex> lck(mutex);
   // Only do queueCleaning if we are adding a video keyframe, so that we guarantee that there is one.
   // No good.  Have to satisfy two conditions:
   // 1. packetqueue starts with a video keyframe
   // 2. Have minimum # of video packets
-  // 3. No packets can be locked
-  // 4. No iterator can point to one of the packets
+  // 3. No iterator can point to one of the packets
   //
   // So start at the beginning, counting video packets until the next keyframe.
   // Then if deleting those packets doesn't break 1 and 2, then go ahead and delete them.
@@ -252,7 +251,6 @@ void PacketQueue::clearPackets(const std::shared_ptr<ZMPacket> &add_packet) {
          );
     return;
   }
-  std::lock_guard<std::mutex> lck(mutex);
 
   // If analysis_it isn't at the end, we need to keep that many additional packets
   int tail_count = 0;
@@ -263,19 +261,28 @@ void PacketQueue::clearPackets(const std::shared_ptr<ZMPacket> &add_packet) {
   }
   Debug(1, "Tail count is %d, queue size is %zu, video_packets %d", tail_count, pktQueue.size(), packet_counts[video_stream_id]);
 
+  // Find the earliest queue_index that any iterator points to.  Since every
+  // packet now carries a monotonic queue_index assigned at enqueue time, this
+  // single integer comparison per packet replaces the previous per-packet
+  // is_there_an_iterator_pointing_to_packet() scan over all iterators.
+  uint64_t min_iterator_queue_index = UINT64_MAX;
+  for (auto iterators_it = iterators.begin(); iterators_it != iterators.end(); ++iterators_it) {
+    packetqueue_iterator *iterator_it = *iterators_it;
+    if (*iterator_it != pktQueue.end()) {
+      uint64_t qi = (*(*iterator_it))->queue_index;
+      if (qi < min_iterator_queue_index)
+        min_iterator_queue_index = qi;
+    }
+  }
+
   if (!keep_keyframes) {
     int packets_removed = 0;
     Debug(3, "Not keeping keyframes");
     // If not doing passthrough, we don't care about starting with a keyframe so logic is simpler
     while ((*pktQueue.begin() != add_packet) and (packet_counts[video_stream_id] > pre_event_video_packet_count + tail_count)) {
       std::shared_ptr<ZMPacket> zm_packet = *pktQueue.begin();
-      ZMPacketLock packet_lock(zm_packet);
-      if (!packet_lock.trylock()) {
-        Debug(1, "Failed locking packet %d", zm_packet->image_index);
-        break;
-      }
 
-      if (is_there_an_iterator_pointing_to_packet(zm_packet)) {
+      if (zm_packet->queue_index >= min_iterator_queue_index) {
         Debug(1, "Found iterator at beginning of queue.");
         break;
       }
@@ -284,7 +291,7 @@ void PacketQueue::clearPackets(const std::shared_ptr<ZMPacket> &add_packet) {
       pktQueue.pop_front();
       int stream_index = zm_packet->packet ? zm_packet->packet->stream_index : 0;
       packet_counts[stream_index] -= 1;
-      Debug(1,
+      Debug(4,
             "Deleting a packet with stream index:%d image_index:%d with keyframe:%d, video frames in queue:%d max: %d, queuesize:%zu",
             stream_index,
             zm_packet->image_index,
@@ -292,16 +299,18 @@ void PacketQueue::clearPackets(const std::shared_ptr<ZMPacket> &add_packet) {
             packet_counts[video_stream_id],
             pre_event_video_packet_count,
             pktQueue.size());
+      packets_to_destroy.push_back(std::move(zm_packet));
     } // end while
     Debug(3, "Done removing %d packets from queue. packet_counts %d >? pre_event %d + tail %d = %d",
         packets_removed, packet_counts[video_stream_id], pre_event_video_packet_count, tail_count, pre_event_video_packet_count + tail_count);
+    // mutex released here at end of scope, packets destroyed after
     return;
   }
 
   auto it = pktQueue.begin();
   auto next_front = pktQueue.begin();
 
-  // First packet is special because we know it is a video keyframe and only need to check for lock
+  // First packet is special because we know it is a video keyframe and only need to check for iterator
   std::shared_ptr<ZMPacket> zm_packet = *it;
   if (zm_packet == add_packet) {
     Debug(1, "First packet in queue is the analysis packet.");
@@ -311,35 +320,19 @@ void PacketQueue::clearPackets(const std::shared_ptr<ZMPacket> &add_packet) {
   int keyframe_interval_count = 0;
   int video_packets_to_delete = 0;    // This is a count of how many packets we will delete so we know when to stop looking
 
-  {
-    ZMPacketLock packet_lock(zm_packet);
-    if (!packet_lock.trylock()) {
-      Debug(4, "Failed getting lock on first packet");
-      return;
-    }  // end if first packet not locked
-   
-    if (is_there_an_iterator_pointing_to_packet(zm_packet)) {
-      Debug(3, "Found iterator Counted %d video packets. Which would leave %d in packetqueue tail count is %d",
-          video_packets_to_delete, packet_counts[video_stream_id]-video_packets_to_delete, tail_count);
-      //zm_packet->unlock(); not necessary due to RAII
-      return;
-    }
-
-    ++it;
+  if (zm_packet->queue_index >= min_iterator_queue_index) {
+    Debug(3, "Found iterator Counted %d video packets. Which would leave %d in packetqueue tail count is %d",
+        video_packets_to_delete, packet_counts[video_stream_id]-video_packets_to_delete, tail_count);
+    return;
   }
 
+  ++it;
+
   // Since we have many packets in the queue, we should NOT be pointing at end so don't need to test for that
   while (*it != add_packet) {
     zm_packet = *it;
-    {
-      ZMPacketLock packet_lock(zm_packet);
-      if (!packet_lock.trylock()) {
-        Debug(3, "Failed locking packet %d", zm_packet->image_index);
-        break;
-      }
-    }
 
-    if (is_there_an_iterator_pointing_to_packet(zm_packet)) {
+    if (zm_packet->queue_index >= min_iterator_queue_index) {
       Debug(3, "Found iterator Counted %d video packets. Which would leave %d in packetqueue tail count is %d",
           video_packets_to_delete, packet_counts[video_stream_id]-video_packets_to_delete, tail_count);
       break;
@@ -377,7 +370,7 @@ void PacketQueue::clearPackets(const std::shared_ptr<ZMPacket> &add_packet) {
         continue;
       }
 
-      Debug(1,
+      Debug(4,
             "Deleting a packet with stream index:%d image_index:%d with keyframe:%d, video frames in queue:%d max: %d, queuesize:%zu",
             zm_packet->packet->stream_index,
             zm_packet->image_index,
@@ -387,31 +380,31 @@ void PacketQueue::clearPackets(const std::shared_ptr<ZMPacket> &add_packet) {
             pktQueue.size());
       pktQueue.pop_front();
       packet_counts[zm_packet->packet->stream_index] -= 1;
+      packets_to_destroy.push_back(std::move(zm_packet));
     }
   }  // end if have at least max_video_packet_count video packets remaining
 
-  return;
-} // end voidPacketQueue::clearPackets(ZMPacket* zm_packet)
+  } // end scope for unique_lock — mutex released here
+  // packets_to_destroy goes out of scope here, destroying packets without holding the mutex
+} // end void PacketQueue::clearPackets(ZMPacket* zm_packet)
 
 void PacketQueue::stop() {
-  {
-    std::lock_guard<std::mutex> lck(mutex);
-    deleting = true;
-    condition.notify_all();
-  }
+  std::lock_guard<std::mutex> lck(mutex);
+  deleting = true;
+  condition.notify_all();
   for (const auto &p : pktQueue) {
     p->notify_all();
   }
 }
 
 void PacketQueue::clear() {
+  Debug(1, "Clearing packetqueue");
+  std::lock_guard<std::mutex> lck(mutex);
   deleting = true;
   // Why are we notifying?
   condition.notify_all();
   if (!packet_counts) // special case, not initialised
     return;
-  Debug(1, "Clearing packetqueue");
-  std::lock_guard<std::mutex> lck(mutex);
 
   while (!pktQueue.empty()) {
     std::shared_ptr<ZMPacket> packet = pktQueue.front();
@@ -439,40 +432,41 @@ void PacketQueue::clear() {
     *iterator_it = pktQueue.begin();
   }  // end foreach iterator
 
-  delete[] packet_counts;
-  packet_counts = nullptr;
+  packet_counts.reset();
   max_stream_id = -1;
   max_keyframe_interval_ = 0;
 
   Debug(1, "Packetqueue is clear, notifying");
   condition.notify_all();
-}
+}  // end void PacketQueue::clear()
 
 unsigned int PacketQueue::size() {
+  std::unique_lock<std::mutex> lck(mutex);
   return pktQueue.size();
 }
 
 int PacketQueue::packet_count(int stream_id) {
+  std::unique_lock<std::mutex> lck(mutex);
   if (stream_id < 0 or stream_id > max_stream_id) {
     Error("Invalid stream_id %d max is %d", stream_id, max_stream_id);
     return -1;
   }
+  if (!packet_counts) {
+    Error("packet_counts not allocated");
+    return -1;
+  }
   return packet_counts[stream_id];
 }  // end int PacketQueue::packet_count(int stream_id)
 
 ZMPacketLock PacketQueue::get_packet_no_wait(packetqueue_iterator *it) {
+  std::unique_lock<std::mutex> lck(mutex);
+  Debug(4, "get_packet_no_wait using it %p queue end? %d",
+      std::addressof(*it), (*it == pktQueue.end()));
   if (deleting or zm_terminate)
     return ZMPacketLock();
-
-  Debug(4, "Locking in get_packet using it %p queue end? %d",
-      std::addressof(*it), (*it == pktQueue.end()));
-
-  // scope for lock
-  std::unique_lock<std::mutex> lck(mutex);
-  Debug(4, "Have Lock in get_packet");
   if ((*it == pktQueue.end()) and !(deleting or zm_terminate)) {
     Debug(2, "waiting.  Queue size %zu it == end? %d", pktQueue.size(), (*it == pktQueue.end()));
-    condition.wait(lck);
+    condition.wait(lck, [&]{ return (*it != pktQueue.end()) || deleting || zm_terminate; });
   }
   if ((*it == pktQueue.end()) or deleting or zm_terminate) return ZMPacketLock();
 
@@ -491,13 +485,11 @@ ZMPacketLock PacketQueue::get_packet(packetqueue_iterator *it) {
   if (deleting or zm_terminate)
     return ZMPacketLock();
 
-  Debug(4, "Locking in get_packet using it %p queue end? %d",
-        std::addressof(*it), (*it == pktQueue.end()));
-
   {
     // scope for lock
     std::unique_lock<std::mutex> lck(mutex);
-    Debug(4, "Have Lock in get_packet");
+    Debug(4, "get_packet using it %p queue end? %d",
+          std::addressof(*it), (*it == pktQueue.end()));
     while (!(deleting or zm_terminate)) {
       while ((*it == pktQueue.end()) and !(deleting or zm_terminate)) {
         Debug(4, "waiting.  Queue size %zu it == end? %d", pktQueue.size(), (*it == pktQueue.end()));
@@ -510,7 +502,7 @@ ZMPacketLock PacketQueue::get_packet(packetqueue_iterator *it) {
         Error("Null p?!");
         return ZMPacketLock();
       }
-      Debug(4, "get_packet using it %p trylocking packet %d", std::addressof(*it), p->image_index);
+      Debug(3, "get_packet using it %p trylocking index %d", std::addressof(*it), p->image_index);
 
       {
         std::shared_ptr<ZMPacket> p = *(*it);
@@ -536,18 +528,15 @@ ZMPacketLock PacketQueue::get_packet_and_increment_it(packetqueue_iterator *it)
   if (deleting or zm_terminate)
     return ZMPacketLock();
 
-  Debug(4, "Locking in get_packet using it %p queue end? %d",
-        std::addressof(*it), (*it == pktQueue.end()));
-
   {
     // scope for lock
     std::unique_lock<std::mutex> lck(mutex);
-    Debug(4, "Have Lock in get_packet");
+    Debug(4, "get_packet_and_increment_it using it %p queue end? %d",
+          std::addressof(*it), (*it == pktQueue.end()));
     while (!(deleting or zm_terminate)) {
       while ((*it == pktQueue.end()) and !(deleting or zm_terminate)) {
         Debug(2, "waiting.  Queue size %zu it == end? %d", pktQueue.size(), (*it == pktQueue.end()));
         condition.wait(lck);
-        Debug(2, "waiting.  Queue size %zu it == end? %d", pktQueue.size(), (*it == pktQueue.end()));
       }
       if (deleting or zm_terminate) break;
 
@@ -556,7 +545,7 @@ ZMPacketLock PacketQueue::get_packet_and_increment_it(packetqueue_iterator *it)
         Error("Null p?!");
         break;
       }
-      Debug(3, "get_packet using it %p locking packet %d", std::addressof(*it), p->image_index);
+      Debug(3, "get_packet using it %p locking index %d", std::addressof(*it), p->image_index);
 
       {
         ZMPacketLock packet_lock(p);
@@ -566,7 +555,7 @@ ZMPacketLock PacketQueue::get_packet_and_increment_it(packetqueue_iterator *it)
           return packet_lock;
         }
       }
-      Debug(2, "waiting.  Queue size %zu it == end? %d, packet %d", pktQueue.size(), (*it == pktQueue.end()), p->image_index);
+      Debug(2, "waiting.  Queue size %zu it == end? %d", pktQueue.size(), (*it == pktQueue.end()));
       condition.wait(lck);
     }  // end while !lp
   }  // end scope for lock
@@ -579,12 +568,21 @@ void PacketQueue::unlock(ZMPacketLock *lp) {
   condition.notify_all();
 }
 
-bool PacketQueue::increment_it(packetqueue_iterator *it) {
-  std::lock_guard<std::mutex> lck(mutex);
+bool PacketQueue::increment_it(packetqueue_iterator *it, bool wait) {
+  std::unique_lock<std::mutex> lck(mutex);
   Debug(2, "Incrementing %p, queue size %zu, end? %d, deleting %d", it, pktQueue.size(), ((*it) == pktQueue.end()), deleting);
-  if (((*it) == pktQueue.end()) or deleting) {
-    Warning("increment_it at end!");
-    return false;
+  if (wait) {
+    while ((*it == pktQueue.end()) and !(deleting or zm_terminate)) {
+      Debug(2, "waiting.  Queue size %zu it == end? %d", pktQueue.size(), (*it == pktQueue.end()));
+      condition.wait(lck);
+      Debug(2, "waiting.  Queue size %zu it == end? %d", pktQueue.size(), (*it == pktQueue.end()));
+    }
+    if (deleting or zm_terminate) return false;
+  } else {
+    if (((*it) == pktQueue.end()) or deleting) {
+      if (!deleting) Debug(1, "increment_it at end!");
+      return false;
+    }
   }
   ++(*it);
   if (*it != pktQueue.end()) {
@@ -597,9 +595,9 @@ bool PacketQueue::increment_it(packetqueue_iterator *it) {
 
 // Increment it only considering packets for a given stream
 bool PacketQueue::increment_it(packetqueue_iterator *it, int stream_id) {
+  std::lock_guard<std::mutex> lck(mutex);
   Debug(2, "Incrementing %p, queue size %zu, end? %d", it, pktQueue.size(), (*it == pktQueue.end()));
 
-  std::lock_guard<std::mutex> lck(mutex);
   if (*it == pktQueue.end()) {
     return false;
   }
@@ -625,6 +623,11 @@ packetqueue_iterator *PacketQueue::get_event_start_packet_it(
   iterators.push_back(it);
 
   *it = snapshot_it;
+  if (*it == pktQueue.end()) {
+    // Should never be able to happen.
+    *it = pktQueue.begin();
+  }
+ 
   std::shared_ptr<ZMPacket> packet = *(*it);
   //ZM_DUMP_PACKET(packet->packet, "snapshot packet");
   // Step one count back pre_event_count frames as the minimum
@@ -645,6 +648,11 @@ packetqueue_iterator *PacketQueue::get_event_start_packet_it(
   }
 
   // it either points to beginning or we have seen pre_event_count video packets.
+  // The loop above doesn't count the begin packet, so check it now.
+  if (pre_event_count and (packet->packet->stream_index == video_stream_id)) {
+    pre_event_count--;
+  }
+
   if (pre_event_count) {
     if (packet->image_index < (int)pre_event_count) {
       // probably just starting up
@@ -668,14 +676,14 @@ packetqueue_iterator *PacketQueue::get_event_start_packet_it(
     packet = *(*it);
   }
   if (!packet->keyframe) {
-    Warning("Hit beginning of packetqueue and packet is not a keyframe. packet is %d", packet->image_index);
+    Warning("Hit beginning of packetqueue and packet is not a keyframe. index is %d", packet->image_index);
   }
   return it;
 }  // end packetqueue_iterator *PacketQueue::get_event_start_packet_it
 
 void PacketQueue::dumpQueue() {
-  std::list<std::shared_ptr<ZMPacket>>::reverse_iterator it;
-  for ( it = pktQueue.rbegin(); it != pktQueue.rend(); ++ it ) {
+  std::lock_guard<std::mutex> lck(mutex);
+  for (auto it = pktQueue.rbegin(); it != pktQueue.rend(); ++it) {
     std::shared_ptr<ZMPacket> zm_packet = *it;
     ZM_DUMP_PACKET(zm_packet->packet, is_there_an_iterator_pointing_to_packet(zm_packet) ? "*" : "");
   }
@@ -685,20 +693,20 @@ void PacketQueue::dumpQueue() {
  * nullptr if no keyframe video packet exists.
  */
 packetqueue_iterator * PacketQueue::get_video_it(bool wait) {
+  std::unique_lock<std::mutex> lck(mutex);
+
   packetqueue_iterator *it = new packetqueue_iterator;
   iterators.push_back(it);
-
-  std::unique_lock<std::mutex> lck(mutex);
   *it = pktQueue.begin();
 
-  if ( wait ) {
+  if (wait) {
     while ( ((! pktQueue.size()) or (*it == pktQueue.end())) and !zm_terminate and !deleting ) {
       Debug(2, "waiting for packets in queue. Queue size %zu it == end? %d", pktQueue.size(), (*it == pktQueue.end()));
       condition.wait(lck);
       *it = pktQueue.begin();
     }
-    if ( deleting or zm_terminate ) {
-      free_it(it);
+    if (deleting or zm_terminate) {
+      iterators.remove(it);
       delete it;
       return nullptr;
     }
@@ -708,7 +716,8 @@ packetqueue_iterator * PacketQueue::get_video_it(bool wait) {
     std::shared_ptr<ZMPacket> zm_packet = *(*it);
     if (!zm_packet) {
       Error("Null zmpacket in queue!?");
-      free_it(it);
+      iterators.remove(it);
+      delete it;
       return nullptr;
     }
     Debug(1, "Packet keyframe %d for stream %d, so returning the it to it",
@@ -759,14 +768,17 @@ bool PacketQueue::is_there_an_iterator_pointing_to_packet(const std::shared_ptr<
 }
 
 void PacketQueue::setMaxVideoPackets(int p) {
+  std::lock_guard<std::mutex> lck(mutex);
   max_video_packet_count = p;
-  Debug(4, "Setting max_video_packet_count to %d", p);
+  Debug(1, "Setting max_video_packet_count to %d", p);
   if (max_video_packet_count < 0)
-    max_video_packet_count = 0 ;
+    max_video_packet_count = 0;
 }
+
 void PacketQueue::setPreEventVideoPackets(int p) {
+  std::lock_guard<std::mutex> lck(mutex);
   pre_event_video_packet_count = p;
-  Debug(4, "Setting pre_event_video_packet_count to %d", p);
+  Debug(1, "Setting pre_event_video_packet_count to %d", p);
   if (pre_event_video_packet_count < 1)
     pre_event_video_packet_count = 1;
   // We can simplify a lot of logic in queuePacket if we can assume at least 1 packet in queue
@@ -780,3 +792,8 @@ void PacketQueue::wait() {
   std::unique_lock<std::mutex> lck(mutex);
   condition.wait(lck);
 }
+
+void PacketQueue::wait_for(Microseconds duration) {
+  std::unique_lock<std::mutex> lck(mutex);
+  condition.wait_for(lck, duration);
+}
