@@ -26,8 +26,8 @@
 
 Storage::Storage() : id(0) {
   Warning("Instantiating default Storage Object. Should not happen.");
-  strcpy(name, "Default");
-  if (staticConfig.DIR_EVENTS[0] != '/') {
+  strncpy(name, "Default", sizeof(name) - 1);
+  if (staticConfig.DIR_EVENTS.empty() || staticConfig.DIR_EVENTS[0] != '/') {
     // not using an absolute path. Make it one by appending ZM_PATH_WEB
     snprintf(path, sizeof(path), "%s/%s",
              staticConfig.PATH_WEB.c_str(), staticConfig.DIR_EVENTS.c_str());
@@ -63,7 +63,7 @@ Storage::Storage(MYSQL_ROW &dbrow) {
 Storage::Storage(unsigned int p_id) : id(p_id) {
   if (id) {
     std::string sql = stringtf("SELECT `Id`, `Name`, `Path`, `Type`, `Scheme` FROM `Storage` WHERE `Id`=%u", id);
-    Debug(4, "Loading Storage for %u using %s", id, sql.c_str());
+    Debug(2, "Loading Storage for %u using %s", id, sql.c_str());
     zmDbRow dbrow;
     if (!dbrow.fetch(sql)) {
       Error("Unable to load storage area for id %d: %s", id, mysql_error(&dbconn));
@@ -81,19 +81,19 @@ Storage::Storage(unsigned int p_id) : id(p_id) {
       } else {
         scheme = SHALLOW;
       }
-      Debug(4, "Loaded Storage area %d '%s'", id, name);
+      Debug(1, "Loaded Storage area %d '%s'", id, name);
     }
   }
   if (!id) {
-    if (staticConfig.DIR_EVENTS[0] != '/') {
+    if (staticConfig.DIR_EVENTS.empty() || staticConfig.DIR_EVENTS[0] != '/') {
       // not using an absolute path. Make it one by appending ZM_PATH_WEB
       snprintf(path, sizeof(path), "%s/%s",
                staticConfig.PATH_WEB.c_str(), staticConfig.DIR_EVENTS.c_str());
     } else {
       strncpy(path, staticConfig.DIR_EVENTS.c_str(), sizeof(path) - 1);
     }
-    Debug(4, "No id passed to Storage constructor.  Using default path %s instead", path);
-    strcpy(name, "Default");
+    Debug(1, "No id passed to Storage constructor.  Using default path %s instead", path);
+    strncpy(name, "Default", sizeof(name) - 1);
     scheme = MEDIUM;
     scheme_str = "Medium";
   }
