@@ -71,22 +71,18 @@ void Zone::Setup(
   overload_count = 0;
   extend_alarm_count = 0;
 
-  // Doing this to reduce shared_ptr accesses which apparently Setup does 16million times because we were using it in forloop
-  unsigned int width = monitor->Width();
-  unsigned int height = monitor->Height();
-
-  pg_image = new Image(width, height, ZM_COLOUR_GRAY8, ZM_SUBPIX_ORDER_NONE);
+  pg_image = new Image(monitor->Width(), monitor->Height(), 1, ZM_SUBPIX_ORDER_NONE);
   pg_image->Clear();
   pg_image->Fill(0xff, polygon);
-  pg_image->Outline(0xff, polygon); // If we just filled it, do we need to outline it?
+  pg_image->Outline(0xff, polygon);
 
-  ranges = new Range[height];
-  for ( unsigned int y = 0; y < height; y++ ) {
+  ranges = new Range[monitor->Height()];
+  for ( unsigned int y = 0; y < monitor->Height(); y++ ) {
     ranges[y].lo_x = -1;
     ranges[y].hi_x = 0;
     ranges[y].off_x = 0;
     const uint8_t *ppoly = pg_image->Buffer( 0, y );
-    for ( unsigned int x = 0; x < width; x++, ppoly++ ) {
+    for ( unsigned int x = 0; x < monitor->Width(); x++, ppoly++ ) {
       if ( *ppoly ) {
         if ( ranges[y].lo_x == -1 ) {
           ranges[y].lo_x = x;
@@ -100,10 +96,13 @@ void Zone::Setup(
 
   if (config.record_diag_images) {
     if (config.record_diag_images_fifo) {
-      diag_path = stringtf("%s/diagpipe-%d-poly.jpg", staticConfig.PATH_SOCKS.c_str(), id);
+      diag_path = stringtf("%s/diagpipe-%d-poly.jpg",
+                           staticConfig.PATH_SOCKS.c_str(), id);
+
       Fifo::fifo_create_if_missing(diag_path);
     } else {
-      diag_path = stringtf("%s/diag-%d-poly.jpg", monitor->getStorage()->Path(), id);
+      diag_path = stringtf("%s/diag-%d-poly.jpg",
+                           monitor->getStorage()->Path(), id);
     }
 
     pg_image->WriteJpeg(diag_path, config.record_diag_images_fifo);
@@ -204,7 +203,7 @@ bool Zone::CheckAlarms(const Image *delta_image) {
 
   if (image)
     delete image;
-  // Get the difference image, using AssignDirect
+  // Get the difference image
   Image *diff_image = image = new Image(*delta_image);
   int diff_width = diff_image->Width();
   uint8_t* diff_buff = diff_image->Buffer();
@@ -736,12 +735,10 @@ bool Zone::CheckAlarms(const Image *delta_image) {
       }  // end for y
 
       if (monitor->Colours() == ZM_COLOUR_GRAY8) {
-        Warning("Highlight RGB when GRAY8");
-        //image = diff_image->HighlightEdges(alarm_rgb, ZM_COLOUR_RGB24, ZM_SUBPIX_ORDER_RGB, &polygon.Extent());
-      //} else {
-        //Warning("Highlight colours %d, %d", monitor->Colours(), monitor->SubpixelOrder());
-      }
+        image = diff_image->HighlightEdges(alarm_rgb, ZM_COLOUR_RGB24, ZM_SUBPIX_ORDER_RGB, &polygon.Extent());
+      } else {
         image = diff_image->HighlightEdges(alarm_rgb, monitor->Colours(), monitor->SubpixelOrder(), &polygon.Extent());
+      }
 
       // Only need to delete this when 'image' becomes detached and points somewhere else
       delete diff_image;
@@ -847,7 +844,7 @@ std::vector<Zone> Zone::Load(const std::shared_ptr<Monitor> &monitor) {
   std::vector<Zone> zones;
   zones.reserve(n_zones);
 
-  for (int i = 0; MYSQL_ROW dbrow = mysql_fetch_row(result); i++) {
+  for (MYSQL_ROW dbrow = mysql_fetch_row(result); dbrow; dbrow = mysql_fetch_row(result)) {
     int col = 0;
 
     int Id = atoi(dbrow[col++]);
@@ -952,12 +949,13 @@ std::vector<Zone> Zone::Load(const std::shared_ptr<Monitor> &monitor) {
   return zones;
 } // end std::vector<Zone> Zone::Load(Monitor *monitor)
 
-bool Zone::DumpSettings(char *output, bool /*verbose*/) const {
-  output[0] = 0;
+std::string Zone::DumpSettings(bool /*verbose*/) const {
+  std::string result;
+  result.reserve(1024);
 
-  sprintf(output+strlen(output), "  Id : %u\n", id );
-  sprintf(output+strlen(output), "  Label : %s\n", label.c_str() );
-  sprintf(output+strlen(output), "  Type: %d - %s\n", type,
+  result += stringtf("  Id : %u\n", id);
+  result += stringtf("  Label : %s\n", label.c_str());
+  result += stringtf("  Type: %d - %s\n", type,
           type==ACTIVE?"Active":(
             type==INCLUSIVE?"Inclusive":(
               type==EXCLUSIVE?"Exclusive":(
@@ -965,28 +963,28 @@ bool Zone::DumpSettings(char *output, bool /*verbose*/) const {
                   type==INACTIVE?"Inactive":(
                     type==PRIVACY?"Privacy":"Unknown"
                   ))))));
-  sprintf( output+strlen(output), "  Shape : %zu points\n", polygon.GetVertices().size() );
+  result += stringtf("  Shape : %zu points\n", polygon.GetVertices().size());
   for (size_t i = 0; i < polygon.GetVertices().size(); i++) {
-    sprintf(output + strlen(output), "  %zu: %d,%d\n", i, polygon.GetVertices()[i].x_, polygon.GetVertices()[i].y_);
+    result += stringtf("  %zu: %d,%d\n", i, polygon.GetVertices()[i].x_, polygon.GetVertices()[i].y_);
   }
-  sprintf( output+strlen(output), "  Alarm RGB : %06x\n", alarm_rgb );
-  sprintf( output+strlen(output), "  Check Method: %d - %s\n", check_method,
+  result += stringtf("  Alarm RGB : %06x\n", alarm_rgb);
+  result += stringtf("  Check Method: %d - %s\n", check_method,
            check_method==ALARMED_PIXELS?"Alarmed Pixels":(
              check_method==FILTERED_PIXELS?"FilteredPixels":(
                check_method==BLOBS?"Blobs":"Unknown"
              )));
-  sprintf( output+strlen(output), "  Min Pixel Threshold : %d\n", min_pixel_threshold );
-  sprintf( output+strlen(output), "  Max Pixel Threshold : %d\n", max_pixel_threshold );
-  sprintf( output+strlen(output), "  Min Alarm Pixels : %d\n", min_alarm_pixels );
-  sprintf( output+strlen(output), "  Max Alarm Pixels : %d\n", max_alarm_pixels );
-  sprintf(output+strlen(output), "  Filter Box : %d,%d\n", filter_box.x_, filter_box.y_ );
-  sprintf( output+strlen(output), "  Min Filter Pixels : %d\n", min_filter_pixels );
-  sprintf( output+strlen(output), "  Max Filter Pixels : %d\n", max_filter_pixels );
-  sprintf( output+strlen(output), "  Min Blob Pixels : %d\n", min_blob_pixels );
-  sprintf( output+strlen(output), "  Max Blob Pixels : %d\n", max_blob_pixels );
-  sprintf( output+strlen(output), "  Min Blobs : %d\n", min_blobs );
-  sprintf( output+strlen(output), "  Max Blobs : %d\n", max_blobs );
-  return true;
+  result += stringtf("  Min Pixel Threshold : %d\n", min_pixel_threshold);
+  result += stringtf("  Max Pixel Threshold : %d\n", max_pixel_threshold);
+  result += stringtf("  Min Alarm Pixels : %d\n", min_alarm_pixels);
+  result += stringtf("  Max Alarm Pixels : %d\n", max_alarm_pixels);
+  result += stringtf("  Filter Box : %d,%d\n", filter_box.x_, filter_box.y_);
+  result += stringtf("  Min Filter Pixels : %d\n", min_filter_pixels);
+  result += stringtf("  Max Filter Pixels : %d\n", max_filter_pixels);
+  result += stringtf("  Min Blob Pixels : %d\n", min_blob_pixels);
+  result += stringtf("  Max Blob Pixels : %d\n", max_blob_pixels);
+  result += stringtf("  Min Blobs : %d\n", min_blobs);
+  result += stringtf("  Max Blobs : %d\n", max_blobs);
+  return result;
 }
 
 void Zone::std_alarmedpixels(
