@@ -8,7 +8,7 @@ function zm_setcookie($cookie, $value, $options=array()) {
     $options['expires'] = time()+3600*24*30*12*10; // 10 years?!
   }
   if (!isset($options['samesite'])) {
-    $options['samesite'] = 'Strict';
+    #$options['samesite'] = 'Strict';
   }
 
   if (version_compare(phpversion(), '7.3.0', '>=')) {
@@ -32,14 +32,14 @@ function zm_session_start() {
     if ( version_compare(phpversion(), '7.3.0', '<') ) {
       session_set_cookie_params(
         $currentCookieParams['lifetime'],
-        $currentCookieParams['path'].'; samesite=strict',
+        $currentCookieParams['path'], #.'; samesite=strict',
         $currentCookieParams['domain'],
         $currentCookieParams['secure'],
         $currentCookieParams['httponly']
       );
     } else {
       # samesite was introduced in 7.3.0
-      $currentCookieParams['samesite'] = 'Strict';
+      #$currentCookieParams['samesite'] = 'Strict';
       session_set_cookie_params($currentCookieParams);
     }
 
